'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fromBase64url = fromBase64url;
exports.toBase64Url = toBase64Url;
exports.encode = encode;
exports.decode = decode;
exports.validate = validate;

var _urlsafeBase = require('urlsafe-base64');

var _urlsafeBase2 = _interopRequireDefault(_urlsafeBase);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Converts a string from a base64url-encoded string
 * @param {string} str
 * @returns {string}
 */
function fromBase64url(str) {
  return _urlsafeBase2.default.decode(str).toString();
}

/**
 * Converts a string to a base64url-encoded string. It also accepts a buffer
 * @param {string|buffer} str
 * @returns {string}
 */
/*!
 * Copyright (c) 2015-2019 Cisco Systems, Inc. See LICENSE file.
 */

function toBase64Url(str) {
  var buffer = str;

  if (!Buffer.isBuffer(buffer)) {
    buffer = Buffer.from(buffer);
  }

  return _urlsafeBase2.default.encode(buffer);
}

/**
 * Converts a string to a base64url-encoded string. It also accepts a buffer
 * @param {string|buffer} str
 * @returns {string}
 */
function encode(str) {
  return toBase64Url(str);
}

/**
 * Converts a string from a base64url-encoded string
 * @param {string} str
 * @returns {string}
 */
function decode(str) {
  return fromBase64url(str);
}

/**
 * Indicates if the provided string is, in fact, a base64 string
 * @param {String} str
 * @returns {Boolean}
 */
function validate(str) {
  return _urlsafeBase2.default.validate(str);
}

exports.default = {
  fromBase64url: fromBase64url,
  toBase64Url: toBase64Url,
  encode: encode,
  decode: decode,
  validate: validate
};
//# sourceMappingURL=base64.js.map
