"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

exports.default = ensureBuffer;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*!
 * Copyright (c) 2015-2019 Cisco Systems, Inc. See LICENSE file.
 */

/* eslint-env: browser */
/* global FileReader */

/**
 * Ensures the provider buffer is, indeed, an ArrayBuffer; converts File and
 * Blob objects to ArrayBuffers.
 * @param {mixed} buffer
 * @returns {Promise<ArrayBuffer>}
 */
function ensureBuffer(buffer) {
  if (buffer instanceof ArrayBuffer) {
    return _promise2.default.resolve(buffer);
  }

  if (buffer.toArrayBuffer) {
    return _promise2.default.resolve(buffer.toArrayBuffer());
  }

  if (buffer.buffer) {
    return _promise2.default.resolve(buffer.buffer);
  }

  return new _promise2.default(function (resolve, reject) {
    var fr = new FileReader();

    fr.onload = function onload() {
      resolve(new Uint8Array(this.result));
    };

    fr.onerror = reject;

    fr.readAsArrayBuffer(buffer);
  });
}
//# sourceMappingURL=ensure-buffer.browser.js.map
