import {isString} from 'lodash';

import {MEETINGS} from '../constants';
import ParameterError from '../common/errors/parameter';

/**
 * @class Collection
 */
export default class Collection {
  namespace = MEETINGS;

  /**
   * @param {String} property
   * @memberof Collection
   */
  constructor(property) {
    if (!property || !isString(property)) {
      throw new ParameterError('Collection expects a string to use as a property name.');
    }
    /**
     * The property name for what collection will be stored, i.e., this.meetingInfos, this.meetings, etc.
     * @instance
     * @type {String}
     * @public
     * @memberof Meetings
     */
    this.propertyName = property;
    /**
     * The actual object for the collection
     * @instance
     * @type {Object}
     * @public
     * @memberof Meetings
     */
    this[this.propertyName] = {};
  }

  /**
   * @param {String} id ID of the thing stuffed into the collection at id location
   * @returns {Any} returns whatever is being stuffed into the collection
   * @public
   * @memberof Collection
   */
  get(id) {
    return this[this.propertyName] && this[this.propertyName][id] ? this[this.propertyName][id] : null;
  }

  /**
   * @param {String} id the id of the meeting info instance to add to the collection
   * @param {Any} value the thing to set in the collection
   * @returns {Any} returns the thing just put in the collection
   * @public
   * @memberof Collection
   */
  set(id, value) {
    this[this.propertyName][id] = value;

    return this.get(id);
  }

  /**
   * remove the thing at the id
   * @param {String} id ID of the thing you wish to delete from the collection
   * @returns {undefined}
   * @public
   * @memberof Collection
   */
  delete(id) {
    delete this[this.propertyName][id];
  }

  /**
   * @returns {Object} returns an object map of things stuffed into the collection
   * @public
   * @memberof Collection
   */
  getAll() {
    return this[this.propertyName];
  }

  /**
   * @param {Object} set the replacement object
   * @returns {Object} returns an object map of things stuffed into the collection
   * @public
   * @memberof Collection
   */
  setAll(set) {
    this[this.propertyName] = set;

    return this.getAll();
  }
}
