import {
  MEETINGS,
  QUALITY_LEVELS
} from '../constants';

import MediaUtil from './util';

/**
 * @class MediaProperties
 */
export default class MediaProperties {
  namespace = MEETINGS;

  /**
   * @param {Object} [options] -- to auto construct
   * @returns {MediaProperties}
   */
  constructor(options = {}) {
    this.peerConnection = MediaUtil.createPeerConnection();
    this.mediaDirection = options.mediaDirection;
    this.videoTrack = options.videoTrack;
    this.audioTrack = options.audioTrack;
    this.shareTrack = options.shareTrack;
    this.remoteStream = options.remoteStream;
    this.remoteShare = options.remoteShare;
    this.remoteQualityLevel = options.remoteQualityLevel || QUALITY_LEVELS.HIGH;
    this.mediaSettings = {};
  }

  setMediaDirection(mediaDirection) {
    this.mediaDirection = mediaDirection;
  }

  setMediaSettings(type, values) {
    this.mediaSettings[type] = values;
  }

  setMediaPeerConnection(peerConnection) {
    this.peerConnection = peerConnection;
  }

  setLocalVideoTrack(videoTrack) {
    this.videoTrack = videoTrack;
  }

  setLocalAudioTrack(audioTrack) {
    this.audioTrack = audioTrack;
  }

  setLocalShareTrack(shareTrack) {
    this.shareTrack = shareTrack;
  }

  setRemoteQualityLevel(remoteQualityLevel) {
    this.remoteQualityLevel = remoteQualityLevel;
  }

  setRemoteStream(remoteStream) {
    this.remoteStream = remoteStream;
  }

  setRemoteShare(remoteShare) {
    this.remoteShare = remoteShare;
  }

  unsetPeerConnection() {
    this.peerConnection = null;
  }

  reInitiatePeerconnection() {
    this.peerConnection = MediaUtil.createPeerConnection();
  }

  unsetLocalVideoTrack() {
    this.videoTrack = null;
  }

  unsetLocalShareTrack() {
    this.shareTrack = null;
  }

  unsetLocalAudioTrack() {
    this.audioTrack = null;
  }

  unsetRemoteStream() {
    this.remoteStream = null;
  }

  unsetRemoteShare() {
    this.remoteShare = null;
  }

  unsetLocalVideoTracks() {
    this.unsetLocalVideoTrack();
    this.unsetLocalShareTrack();
  }

  unsetRemoteStreams() {
    this.unsetRemoteStream();
    this.unsetRemoteShare();
  }

  unsetShareStreams() {
    this.unsetLocalShareTrack();
    this.unsetRemoteShare();
  }

  unsetMediaStreams() {
    this.unsetLocalVideoTrack();
    this.unsetRemoteStream();
  }
}
