import sinon from 'sinon';
import {assert} from '@webex/test-helper-chai';
import MockWebex from '@webex/test-helper-mock-webex';
import Meetings from '@webex/plugin-meetings';
import MeetingRequest from '@webex/plugin-meetings/src/meeting/request';

describe('plugin-meetings', () => {
  let meetingsRequest;

  beforeEach(() => {
    const webex = new MockWebex({
      children: {
        meetings: Meetings
      }
    });

    meetingsRequest = new MeetingRequest({}, {
      parent: webex
    });

    meetingsRequest.request = sinon.mock().returns(Promise.resolve({}));
  });

  describe('meeting request library', () => {
    describe('#sendDTMF', () => {
      it('sends a POST to the sendDtmf locus endpoint', async () => {
        const locusUrl = 'locusURL';
        const deviceUrl = 'deviceUrl';
        const tones = '1234';

        await meetingsRequest.sendDTMF({
          locusUrl,
          deviceUrl,
          tones
        });
        const requestParams = meetingsRequest.request.getCall(0).args[0];

        assert.equal(requestParams.method, 'POST');
        assert.equal(requestParams.uri, `${locusUrl}/sendDtmf`);
        assert.equal(requestParams.body.dtmf.tones, tones);
        assert.equal(requestParams.body.deviceUrl, deviceUrl);
      });
    });
  });
});
